package org.processmininguc.balsa.models.impl.smadu;

import java.util.List;

import org.processmininguc.balsa.models.StateMachineDiagramCondition;
import org.processmininguc.balsa.models.StateMachineDiagramEffect;
import org.processmininguc.balsa.models.StateMachineDiagramEvent;
import org.processmininguc.balsa.models.StateMachineDiagramState;
import org.processmininguc.balsa.models.StateMachineDiagramTransition;
import org.processmininguc.parser.GuardProcessor;
import org.processmininguc.parser.ProcessedGuard;

import stateMachine.Effect;
import stateMachine.Event;
import stateMachine.FinalState;
import stateMachine.Pseudostate;
import stateMachine.State;
import stateMachine.Transition;
import stateMachine.Trigger;
import stateMachine.Vertex;

public class StateMachineDiagramTransitionImpl extends Transition implements StateMachineDiagramTransition {
	
	/*Derived attribute from the condition in the superclass. This is necessary because in some cases it may be a condition
	 * referring to the number of artifacts. StateMachineDaigramCondImpl has a subclass which is precisely in charge of storing
	 * the details.
	 */
	private StateMachineDiagramCondImpl m_condGuard;
	
	protected StateMachineDiagramTransitionImpl(){
		super();
		m_condGuard = null;
	}
	
	protected StateMachineDiagramTransitionImpl(Vertex source, Vertex target){
		super(source,target);
		m_condGuard = null;
	}
	
	protected StateMachineDiagramTransitionImpl(Vertex source, Vertex target, List<Trigger> triggerList, String guard,
			EventDepCondType evtDepCond, Effect effect) {
		super(source,target,triggerList,guard,evtDepCond,effect);
		ProcessedGuard pg = GuardProcessor.processGuard(guard);
		if (pg.isSucessful()){
			StateMachineDiagramArtCondImpl artCond = new StateMachineDiagramArtCondImpl(guard, pg.getArtName(), pg.getComparisonType(), pg.getNumber());
			m_condGuard = artCond;
		}
		else {
			//Print reason why it cannot be processed?
		}
	}


	/**
	 * A transition is initial if it has an initial pseudostate as source node.
	 */
	public boolean isInitial() {
		Vertex v = super.getSource();
		return v.isInitialPseudostate();
	}

	/**
	 * A transition is final if its target node is a final node.
	 */
	public boolean isFinal() {
		Vertex v = super.getTarget();
		return v.isFinalState();
	}
	
	/**
	 * Returns the source node of the transition. If it is different from a state or an initial node, it returns null.
	 */
	public StateMachineDiagramState getSourceNode() {
		Vertex v = getSource();
		StateMachineDiagramState smdState = null;
		if (v.isState()){
			State state = (State)v;
			smdState = (StateMachineDiagramRealStateImpl) state;
		}
		else if (v.isInitialPseudostate()){
			Pseudostate iniState = (Pseudostate) v;
			smdState = (StateMachineDiagramPseudostateImpl) iniState;
		}
		return smdState;
	}

	/**
	 * Returns the target node of the transition. If it is different from a state or a final node, it returns null.
	 */
	public StateMachineDiagramState getTargetNode() {
		Vertex v = getTarget();
		StateMachineDiagramState smdState = null;
		if (v.isState()){
			State state = (State)v;
			smdState = (StateMachineDiagramRealStateImpl) state;
		}
		else if (v.isFinalState()){
			FinalState finalState = (FinalState) v;
			smdState = (StateMachineDiagramFinalStateImpl) finalState;
		}
		return smdState;
	}

	/**
	 * If the state machine diagram has more than one event per transition (or none), it returns null.
	 * Otherwise, it returns the event.
	 */
	public StateMachineDiagramEvent getEvent() {
		List<Event> eventList = super.getEvents();
		if (eventList == null || eventList.size() != 1){
			return null;
		}
		else {
			Event e = eventList.get(0);
			StateMachineDiagramEventImpl smdEvent = (StateMachineDiagramEventImpl) e;
			return smdEvent;
		}
	}

	
	public StateMachineDiagramCondition getGuardCondition() {
//		String cond = super.getGuard();
//		StateMachineDiagramCondImpl smdCond = new StateMachineDiagramCondImpl(cond);
//		return smdCond;
		return m_condGuard;
	}
	
	
	public StateMachineDiagramCondImpl getArtCondition(){
		return m_condGuard;
	}
	
	public void setArtCondition(StateMachineDiagramCondImpl smdArtCond){
		m_condGuard = smdArtCond;
	}
	
	/**
	 * Overrides setGuard from Transition
	 */
	public void setGuard(String guard){
		super.setGuard(guard);
		ProcessedGuard pg = GuardProcessor.processGuard(guard);
		if (pg.isSucessful()){
			StateMachineDiagramArtCondImpl artCond = new StateMachineDiagramArtCondImpl(guard, pg.getArtName(), pg.getComparisonType(), pg.getNumber());
			m_condGuard = artCond;
		}
		else {
			//Print reason why it cannot be processed?
		}
	}
	
	/**
	 * To be able to group transitions according to the events that make them up.
	 */
	public int compareTo(StateMachineDiagramTransition smdT){
		//We order by event first and then by guard condition. We do not consider event-dependent conditions (i.e. tags)
		StateMachineDiagramEvent smdEvent = smdT.getEvent();
		if (this.getEvent() == null || smdEvent == null){
			if(this.getEvent() == null && smdEvent == null){
				return 0;
			}
			else if (this.getEvent() == null) {
				return 1;
 			}
			else {
				// smdT.getEvent is null
				return -1;
			}
		}
		String eventName = smdEvent.getName();
		String thisEventName = this.getEvent().getName();
		int result;
		if (eventName.equals(thisEventName)){
			String condition = null;
			String thisCondition = null;
			if(smdT.getGuardCondition() != null){
				condition = smdT.getGuardCondition().getCondition();
			}
			if(smdT.getGuardCondition() != null){
				thisCondition = getGuardCondition().getCondition();
			}
			if(condition!= null && thisCondition!=null){
				result = thisCondition.compareTo(condition);
			}
			else if(condition != null){
				result = 1;
			}
			else if(thisCondition != null){
				result = -1;
			}
			else { //Both will be null
				result = 0;
			}
		}
		else{
			result = thisEventName.compareTo(eventName);
		}
		return result;
	}

	public EventDepCondType getTag() {
		return getCond();
	}
	
	public StateMachineDiagramEffect getSmdEffect(){
		StateMachineDiagramEffectImpl ef = (StateMachineDiagramEffectImpl) super.getEffect();
		return ef;
	}





}
